/*
 * gtk3-menu-shell.cc - Copyright (c) 2001-2024 - Olivier Poncet
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <cerrno>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cstdint>
#include <climits>
#include <cassert>
#include <memory>
#include <string>
#include <vector>
#include <iostream>
#include <stdexcept>
#include "gtk3-menu-shell.h"

// ---------------------------------------------------------------------------
// gtk3::MenuShellTraits
// ---------------------------------------------------------------------------

namespace gtk3 {

struct MenuShellTraits
    : BasicTraits
{
    static GtkWidget* create_menu_shell()
    {
        return nullptr;
    }

    static void append(MenuShell& menu_shell, Widget& widget)
    {
        if(menu_shell && widget) {
            ::gtk_menu_shell_append(menu_shell, widget);
        }
    }
};

}

// ---------------------------------------------------------------------------
// <anonymous>::traits
// ---------------------------------------------------------------------------

namespace {

using traits = gtk3::MenuShellTraits;

}

// ---------------------------------------------------------------------------
// gtk3::MenuShell
// ---------------------------------------------------------------------------

namespace gtk3 {

MenuShell::MenuShell()
    : MenuShell(traits::create_menu_shell())
{
}

MenuShell::MenuShell(GtkWidget* instance)
    : Container(instance)
{
}

void MenuShell::append(Widget& widget)
{
    return traits::append(*this, widget);
}

}

// ---------------------------------------------------------------------------
// End-Of-File
// ---------------------------------------------------------------------------
